package cz.kiele.homeautomation.core.model;

import cz.kiele.homeautomation.core.controller.AutomationController;
import cz.kiele.homeautomation.core.converter.LocalDateTimeConverter;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.time.LocalDateTime;

@Entity
@Table(name = "event")
public class Event {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @NotNull
    @Column(name = "event_id", unique = true)
    private long id;

    @NotNull
    private String command;

    private String commandParameter;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "actuator_id", nullable = false)
    private Actuator actuator;

    @NotNull
    @Column(updatable = false)
    @Convert(converter = LocalDateTimeConverter.class)
    private LocalDateTime created;

    public Event() {
        this.created = LocalDateTime.now();
    }


    public Event(String command) {
        this.command = command;
        this.created = LocalDateTime.now();
    }

    public Event(String command, String commandParameter) {
        this.command = command;
        this.commandParameter = commandParameter;
        this.created = LocalDateTime.now();
    }

    public Event(String command, Actuator actuator) {
        this.command = command;
        this.actuator = actuator;
        this.created = LocalDateTime.now();
    }

    public Event(String command, String commandParameter, Actuator actuator) {
        this.command = command;
        this.commandParameter = commandParameter;
        this.actuator = actuator;
        this.created = LocalDateTime.now();
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getCommand() {
        return command;
    }

    public void setCommand(String command) {
        this.command = command;
    }

    public String getCommandParameter() {
        return commandParameter;
    }

    public void setCommandParameter(String commandParameter) {
        this.commandParameter = commandParameter;
    }

    public Actuator getActuator() {
        return actuator;
    }

    public void setActuator(Actuator actuator) {
        this.actuator = actuator;
    }

    public LocalDateTime getCreated() {
        return created;
    }

    public void setCreated(LocalDateTime created) {
        this.created = created;
    }

    public void execute(AutomationController controller, ActuatorState actuatorState){
        try {
            Class params[] = {Actuator.class, ActuatorState.class};
            Class cls = controller.getClass();
            Method method = cls.getDeclaredMethod(this.command, params);
            method.invoke(controller, this.actuator, actuatorState);
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }

    }
}
