package cz.kiele.homeautomation.core.model;

import cz.kiele.homeautomation.core.converter.LocalDateTimeConverter;
import cz.kiele.homeautomation.core.enums.Unit;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

@Entity
@Table(name = "sensor_data")
public class SensorData {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @NotNull
    @Column(name = "sensor_data_id", unique = true)
    private Long id;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "sensor_id", nullable = false)
    private Sensor sensor;

    @NotNull
    private float value;

    @Enumerated(EnumType.STRING)
    private Unit unit;

    @NotNull
    @Column(updatable = false)
    @Convert(converter = LocalDateTimeConverter.class)
    private LocalDateTime timestamp;

    public SensorData() {
       this.timestamp = LocalDateTime.now();
    }

    public SensorData(Sensor sensor, float value) {
        this.sensor = sensor;
        this.value = value;
        this.timestamp = LocalDateTime.now();
    }

    public SensorData(Sensor sensor, float value, Unit unit) {
        this.sensor = sensor;
        this.value = value;
        this.unit = unit;
        this.timestamp = LocalDateTime.now();
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Sensor getSensor() {
        return sensor;
    }

    public void setSensor(Sensor sensor) {
        this.sensor = sensor;
    }

    public float getValue() {
        return value;
    }

    public void setValue(float value) {
        this.value = value;
    }

    public Unit getUnit() {
        return unit;
    }

    public void setUnit(Unit unit) {
        this.unit = unit;
    }

    public LocalDateTime getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(LocalDateTime timestamp) {
        this.timestamp = timestamp;
    }
}
