package cz.kiele.homeautomation.eventmanager.manager;

import cz.kiele.homeautomation.core.dao.ActuatorStateDao;
import cz.kiele.homeautomation.core.dao.EventDao;
import cz.kiele.homeautomation.core.dao.RuleDao;
import cz.kiele.homeautomation.core.model.ActuatorState;
import cz.kiele.homeautomation.core.model.Event;
import cz.kiele.homeautomation.core.model.Rule;
import cz.kiele.homeautomation.eventmanager.mapper.CommandToNewStateMap;
import cz.kiele.homeautomation.eventmanager.mapper.RuleExpressionMapper;
import org.apache.commons.jexl3.JexlBuilder;
import org.apache.commons.jexl3.JexlContext;
import org.apache.commons.jexl3.JexlEngine;
import org.apache.commons.jexl3.JexlExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

@Component("ruleManager")
public class RuleManager {

    @Autowired
    private RuleDao ruleDao;

    @Autowired
    private EventDao eventDao;

    @Autowired
    private ActuatorStateDao actuatorStateDao;

    @Autowired
    private RuleExpressionMapper mapper;


    private static final JexlEngine jexl = new JexlBuilder().cache(512).strict(true).silent(false).create();

    public void evaluateAllRules(){
        List<Rule> rules = ruleDao.findAll();
        for (Rule rule: rules) {
            ActuatorState state = actuatorStateDao.findByActuatorAndName(rule.getActuator(),
                    CommandToNewStateMap.commandToNewStateMap.get(rule.getCommand()).getName());
            if(evaluateRule(rule) && evaluateActuatorStateChange(rule, state)){
                Event event = new Event(rule.getCommand(), rule.getCommandParameter(), rule.getActuator());
                eventDao.save(event);
                actuatorStateDao.save(prepareNewState(rule));
            }
        }
    }

    private boolean evaluateRule(Rule rule){
        JexlExpression expression = jexl.createExpression( rule.getExpression() );

        // populate the context
        JexlContext context = mapper.createJexlContext(rule);
        Boolean result = (Boolean) expression.evaluate(context);

        return result;
    }

    private boolean evaluateActuatorStateChange(Rule rule, ActuatorState state){
        return !state.getValue().equals(CommandToNewStateMap.commandToNewStateMap.get(rule.getCommand()).getValue());
    }

    private ActuatorState prepareNewState(Rule rule){
        String stateName = CommandToNewStateMap.commandToNewStateMap.get(rule.getCommand()).getName();
        ActuatorState newState = actuatorStateDao.findByActuatorAndNameAndIsNew(rule.getActuator(), stateName, true);
        if (newState == null) {
            newState = CommandToNewStateMap.commandToNewStateMap.get(rule.getCommand());
        }
        newState.setActuator(rule.getActuator());
        newState.setName(stateName);
        newState.setValue(CommandToNewStateMap.commandToNewStateMap.get(rule.getCommand()).getValue());
        if (newState.getValue() == null){
            newState.setValue(rule.getCommandParameter());
        }
        newState.setNew(true);
        return newState;
    }

}
