package cz.kiele.homeautomation.eventmanager.quartz;

import cz.kiele.homeautomation.core.controller.AutomationController;
import cz.kiele.homeautomation.core.dao.ActuatorStateDao;
import cz.kiele.homeautomation.core.dao.EventDao;
import cz.kiele.homeautomation.core.enums.ActuatorType;
import cz.kiele.homeautomation.core.model.Actuator;
import cz.kiele.homeautomation.core.model.ActuatorState;
import cz.kiele.homeautomation.core.model.Event;
import cz.kiele.homeautomation.eventmanager.controller.MasterController;
import cz.kiele.homeautomation.eventmanager.mapper.CommandToNewStateMap;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Component;

import java.util.List;

@Component("eventExecutionerJob")
public class EventExecutionerJob extends QuartzJobBean {

    @Autowired
    private ActuatorStateDao actuatorStateDao;

    @Autowired
    private EventDao eventDao;

    @Autowired
    private MasterController masterController;

    @Override
    protected void executeInternal(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        List<Event> events = eventDao.findAllByOrderByCreatedDesc();
        for (Event event : events) {
            actuatorStateDao.save(prepareNewState(event));
            ActuatorState newState = actuatorStateDao.findByActuatorAndNameAndIsNew(event.getActuator(),
                    CommandToNewStateMap.commandToNewStateMap.get(event.getCommand()).getName(), true);
            if (newState != null) {
                event.execute(getControllerForActuator(event.getActuator()), newState);
            }
            eventDao.delete(event);
        }
    }

    private AutomationController getControllerForActuator(Actuator actuator){
        if(actuator.getType().equals(ActuatorType.LIGHT) || actuator.getType().equals(ActuatorType.LIGHT_BULB)){
            return masterController.getLightController();
        }
        return null;
    }

    private ActuatorState prepareNewState(Event event){
        String stateName = CommandToNewStateMap.commandToNewStateMap.get(event.getCommand()).getName();
        ActuatorState newState = actuatorStateDao.findByActuatorAndNameAndIsNew(event.getActuator(), stateName, true);
        if (newState == null) {
            newState = CommandToNewStateMap.commandToNewStateMap.get(event.getCommand());
        }
        newState.setActuator(event.getActuator());
        newState.setName(stateName);
        newState.setValue(CommandToNewStateMap.commandToNewStateMap.get(event.getCommand()).getValue());
        if (newState.getValue() == null){
            newState.setValue(event.getCommandParameter());
        }
        newState.setNew(true);
        return newState;
    }
}
