package cz.kiele.homeautomation.lightmodule.controller;

import cz.kiele.homeautomation.core.dao.ActuatorStateDao;
import cz.kiele.homeautomation.core.model.Actuator;
import cz.kiele.homeautomation.core.adapters.LightControlAdapter;
import cz.kiele.homeautomation.core.model.ActuatorState;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class LightControllerImpl implements LightController{

    @Autowired
    private LightControlAdapter lightControlAdapter;

    @Autowired
    private ActuatorStateDao stateDao;

    public void turnLightOn(Actuator actuator, ActuatorState newState){
        ActuatorState state = prepareState(actuator, newState);
        lightControlAdapter.turnLightOn(actuator);
        stateDao.save(state);
    }

    public void turnLightOff(Actuator actuator, ActuatorState newState){
        ActuatorState state = prepareState(actuator, newState);
        lightControlAdapter.turnLightOff(actuator);
        stateDao.save(state);
    }

    public void setBrightness(Actuator actuator, ActuatorState newState){
        ActuatorState state = prepareState(actuator, newState);
        lightControlAdapter.setBrightness(actuator, Double.parseDouble(newState.getValue()));
        stateDao.save(state);
    }

    private ActuatorState prepareState(Actuator actuator, ActuatorState newState){
        ActuatorState currentState = stateDao.findByActuatorAndNameAndIsNew(actuator, newState.getName(), false);
        if (currentState == null){
            currentState = new ActuatorState();
            currentState.setName(newState.getName());
            currentState.setActuator(actuator);
        }
        currentState.setValue(newState.getValue());
//        if
        ActuatorState stateToDelete = stateDao.findByActuatorAndNameAndIsNew(actuator, newState.getName(), true);
        if (stateToDelete != null) {
            stateDao.delete(stateToDelete);
        }
        return currentState;
    }
}
