package cz.kiele.homeautomation.restapi.controller;

import cz.kiele.homeautomation.core.dao.RoomDao;
import cz.kiele.homeautomation.core.model.Event;
import cz.kiele.homeautomation.core.model.Room;
import cz.kiele.homeautomation.restapi.data_objects.RoomDO;
import cz.kiele.homeautomation.restapi.service.ActuatorService;
import cz.kiele.homeautomation.restapi.service.EventService;
import cz.kiele.homeautomation.restapi.service.RoomService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
public class RoomController {

    @Autowired
    RoomService roomService;

    @Autowired
    ActuatorService actuatorService;

    @Autowired
    EventService eventService;

    @Autowired
    RoomDao roomDao;

    @RequestMapping(value = "/room/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<RoomDO> getRoomDetail(@PathVariable("id") int id)
    {
        Room room = roomService.findById(id);
        RoomDO roomDO = roomService.getFilledRoomDO(room);
        return new ResponseEntity<RoomDO>(roomDO, HttpStatus.OK);
    }

    @RequestMapping(value = "/lightOn/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Event> lightOn(@PathVariable("id") String id)
    {
        Event event = new Event("turnLightOn", actuatorService.getActuatorById(id));
        eventService.save(event);
        return new ResponseEntity<Event>(event, HttpStatus.OK);
    }

    @RequestMapping(value = "/lightOff/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Event> lightOff(@PathVariable("id") String id)
    {
        Event event = new Event("turnLightOff", actuatorService.getActuatorById(id));
        eventService.save(event);
        return new ResponseEntity<Event>(event, HttpStatus.OK);
    }

    @RequestMapping(value = "/addRoom}", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Room> addNewRoom(@RequestBody Room room)
    {
        Room newRoom = roomDao.save(room);
        return new ResponseEntity<Room>(newRoom, HttpStatus.OK);
    }


}
