package cz.kiele.homeautomation.restapi.controller;

import cz.kiele.homeautomation.core.dao.*;
import cz.kiele.homeautomation.core.model.*;
import cz.kiele.homeautomation.restapi.data_objects.SetupDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class SetupController {

    @Autowired
    UserDao userDao;

    @Autowired
    DeviceDao deviceDao;

    @Autowired
    RoomDao roomDao;

    @Autowired
    RuleDao ruleDao;

    @Autowired
    SensorDao sensorDao;

    @Autowired
    ActuatorDao actuatorDao;

    @RequestMapping(value = "/getSetup", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<SetupDTO> getSetup()
    {
        SetupDTO setup = new SetupDTO();
        setup.setUsers(userDao.findAll());
        setup.setDevices(deviceDao.findAll());
        setup.setRooms(roomDao.findAll());
        setup.setRules(ruleDao.findAll());
        setup.setSensors(sensorDao.findAll());
        setup.setActuators(actuatorDao.findAll());
        return new ResponseEntity<SetupDTO>(setup, HttpStatus.OK);
    }

    @RequestMapping(value = "/room/delete/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Room> deleteRoom(@PathVariable("id") int id)
    {
        Room room = roomDao.findOne(id);
        roomDao.delete(room);
        return new ResponseEntity<Room>(room, HttpStatus.OK);
    }

    @RequestMapping(value = "/user/delete/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<User> deleteUser(@PathVariable("id") int id)
    {
        User user = userDao.findOne(id);
        userDao.delete(user);
        return new ResponseEntity<User>(user, HttpStatus.OK);
    }

    @RequestMapping(value = "/device/delete/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Device> deleteDevice(@PathVariable("id") int id)
    {
        Device device = deviceDao.findOne(id);
        deviceDao.delete(device);
        return new ResponseEntity<Device>(device, HttpStatus.OK);
    }

    @RequestMapping(value = "/rule/delete/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Rule> deleteRule(@PathVariable("id") long id)
    {
        Rule rule = ruleDao.findOne(id);
        ruleDao.delete(rule);
        return new ResponseEntity<Rule>(rule, HttpStatus.OK);
    }

    @RequestMapping(value = "/sensor/delete/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Sensor> deleteSensor(@PathVariable("id") String id)
    {
        Sensor sensor = sensorDao.findOne(id);
        sensorDao.delete(sensor);
        return new ResponseEntity<Sensor>(sensor, HttpStatus.OK);
    }

    @RequestMapping(value = "/actuator/delete/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Actuator> deleteActuator(@PathVariable("id") String id)
    {
        Actuator actuator = actuatorDao.findOne(id);
        actuatorDao.delete(actuator);
        return new ResponseEntity<Actuator>(actuator, HttpStatus.OK);
    }
}
