package cz.kiele.homeautomation.restapi.service;

import cz.kiele.homeautomation.core.dao.*;
import cz.kiele.homeautomation.core.enums.ActuatorType;
import cz.kiele.homeautomation.core.enums.SensorType;
import cz.kiele.homeautomation.core.enums.Unit;
import cz.kiele.homeautomation.core.model.Actuator;
import cz.kiele.homeautomation.core.model.Room;
import cz.kiele.homeautomation.core.model.Sensor;
import cz.kiele.homeautomation.core.model.SensorData;
import cz.kiele.homeautomation.restapi.data_objects.AllRoomsDO;
import cz.kiele.homeautomation.restapi.data_objects.RoomDO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class RoomServiceImpl implements RoomService{

    @Autowired
    RoomDao roomDao;

    @Autowired
    SensorDao sensorDao;

    @Autowired
    SensorDataDao sensorDataDao;

    @Autowired
    ActuatorDao actuatorDao;

    @Autowired
    ActuatorStateDao actuatorStateDao;

    @Autowired
    RuleDao ruleDao;

    public AllRoomsDO getAllRoomsInfo(){
        List<Room> rooms = roomDao.findAll();
        List<RoomDO> roomsInfo = new ArrayList<>();
        for (Room room : rooms){
            RoomDO roomInfo = getFilledRoomDO(room);
            roomsInfo.add(roomInfo);
        }
        AllRoomsDO allRoomsInfo = new AllRoomsDO();
        allRoomsInfo.setAllRoomsInfo(roomsInfo);
        return allRoomsInfo;
    }

    public RoomDO getFilledRoomDO(Room room){
        RoomDO roomInfo = new RoomDO();
        roomInfo.setRoom(room);
        roomInfo.setTemperature(getAvarageRoomTemperature(room));
        roomInfo.setHumidity(getAvarageRoomHumidity(room));
        roomInfo.setLightState(getLightState(room));
        roomInfo.setRules(ruleDao.findByRoom(room));
        roomInfo.setSensors(sensorDao.findByRoom(room));
        roomInfo.setActuators(actuatorDao.findByRoom(room));
        return roomInfo;
    }

    private float getAvarageRoomTemperature(Room room){
        List<Sensor> sensors = sensorDao.findAllByRoomAndType(room, SensorType.TEMPERATURE_HUMIDITY);
        float avarageTemperature = 0;
        for (Sensor sensor : sensors){
            SensorData lastTemperature = sensorDataDao.findFirstBySensorAndUnitOrderByTimestampDesc(sensor, Unit.CELSIUS);
            avarageTemperature += lastTemperature.getValue();
        }
        return avarageTemperature / sensors.size();
    }

    private float getAvarageRoomHumidity(Room room){
        List<Sensor> sensors = sensorDao.findAllByRoomAndType(room, SensorType.TEMPERATURE_HUMIDITY);
        float avarageHumidity = 0;
        for (Sensor sensor : sensors){
            SensorData lastHumidity = sensorDataDao.findFirstBySensorAndUnitOrderByTimestampDesc(sensor, Unit.HUMIDITY);
            avarageHumidity += lastHumidity.getValue();
        }
        return avarageHumidity / sensors.size();
    }

    private String getLightState(Room room){
        Actuator light = actuatorDao.findByRoomAndType(room, ActuatorType.LIGHT_BULB);
        String state = "NaN";
        if (light != null) {
            state = actuatorStateDao.findByActuatorAndName(light, "power").getValue();
        }
        return state;
    }

    @Override
    public Room findById(int id) {
        return roomDao.findOne(id);
    }
}
