package cz.kiele.homeautomation.transcom.actuators.lifx.communicator;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import cz.kiele.homeautomation.transcom.actuators.lifx.model.Bulb;
import org.springframework.stereotype.Component;

import javax.ws.rs.client.*;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

@Component
public class LifxRestApiCommunicator implements RestApiCommunicator {

    private static final String LIFX_SET_STATE_URL = "https://api.lifx.com/v1/lights/SELECTOR/state";

    private static final String LIFX_GET_STATE_URL = "https://api.lifx.com/v1/lights/SELECTOR";

    private static final String AUTH_TOKEN = "cc1c890e77506b16f8f982208dfc44ac4acc83cb50cd2233a6d1cbfb5ac3be7e";

    private static final String SELECTOR_REPLACE_REGEX = "(SELECTOR)";

    @Override
    public void sendPutJsonRequest(Invocation.Builder request, String jsonPayload) {
        Response response = request.put(Entity.json(jsonPayload));
    }

    public void setLifxBulbState(Bulb bulb){
        Invocation.Builder request = createRequest(mapLifxApiURLWithSelector(LIFX_SET_STATE_URL, bulb.getSelector()));
        String payload = createJsonPayload(bulb);
        sendPutJsonRequest(request, payload);
    }

    public String getLifxBulbState(String selector){
        Invocation.Builder request = createRequest(mapLifxApiURLWithSelector(LIFX_GET_STATE_URL, selector));
        Response response = request.get();
        return response.readEntity(String.class);
    }

    public String createJsonPayload(Bulb bulb){
        ObjectMapper mapper = new ObjectMapper();
        String payload = "";
        try {
            payload = mapper.writeValueAsString(bulb);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        return payload;
    }

    private String mapLifxApiURLWithSelector(String url, String selector){
        return url.replaceAll(SELECTOR_REPLACE_REGEX, selector);
    }

    private Invocation.Builder createRequest(String url){
        Client client = ClientBuilder.newClient();

        WebTarget resource = client.target(url);

        Invocation.Builder request = resource.request();
        request.accept(MediaType.APPLICATION_JSON);
        request.header("Authorization", "Bearer " + AUTH_TOKEN);

        return request;
    }

    public Response sendPutJsonRequest(String url, String jsonPayload) {
        Client client = ClientBuilder.newClient();

        WebTarget resource = client.target(url);

        Invocation.Builder request = resource.request();
        request.accept(MediaType.APPLICATION_JSON);
        request.header("Authorization", "Bearer " + AUTH_TOKEN);

        return request.put(Entity.json(jsonPayload));
    }
}
