package cz.kiele.homeautomation.transcom.controllers;

import cz.kiele.homeautomation.core.model.Sensor;
import cz.kiele.homeautomation.core.model.SensorData;
import cz.kiele.homeautomation.transcom.sensors.collectors.TemperatureAndHumidityDataCollector;
import cz.kiele.homeautomation.transcom.sensors.mappings.SensorCoreModelMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
public class TemperatureAndHumidityDataController {

    Logger logger = LoggerFactory.getLogger(TemperatureAndHumidityDataController.class);

    @Autowired
    private SensorCoreModelMapper mapper;

    @RequestMapping(value = "/humidityAndTemperatureCollect", method = RequestMethod.POST, headers = {"Content-type=application/json"})
    public @ResponseBody
    void collectTempAndHumidity(@RequestBody TemperatureAndHumidityDataCollector temperatureAndHumidityData){
        logger.info(String.format("Receiving data from light sensor with id: %s", temperatureAndHumidityData.getSensorId()) );

        Sensor sensor = (Sensor) mapper.mapSensorToModel(temperatureAndHumidityData.getSensorId());
        SensorData sensorDataTemperature = mapper.createSensorData(sensor, temperatureAndHumidityData.getTemperature(), SensorCoreModelMapper.TEMPERATURE);
        SensorData sensorDataHumidity = mapper.createSensorData(sensor, temperatureAndHumidityData.getHumidity(), SensorCoreModelMapper.HUMIDITY);
        mapper.storeSensorValues(sensorDataTemperature);
        mapper.storeSensorValues(sensorDataHumidity);

        logger.info(String.format("Data from sensor %s stored!", temperatureAndHumidityData.getSensorId()) );
    }
}
