package cz.kiele.homeautomation.transcom.sensors.mappings;

import cz.kiele.homeautomation.core.dao.SensorDao;
import cz.kiele.homeautomation.core.dao.SensorDataDao;
import cz.kiele.homeautomation.core.exception.EmptySensorDataException;
import cz.kiele.homeautomation.core.exception.UnknownSensorException;
import cz.kiele.homeautomation.core.mapping.SensorCoreModelMapper;
import cz.kiele.homeautomation.core.model.Sensor;
import cz.kiele.homeautomation.core.model.SensorData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public abstract class AbstractSensorMapper implements SensorCoreModelMapper {

    Logger logger = LoggerFactory.getLogger(AbstractSensorMapper.class);

    @Autowired
    SensorDao sensorDao;

    @Autowired
    SensorDataDao sensorDataDao;

    @Override
    public cz.kiele.homeautomation.core.model.Component mapSensorToModel(String sensorId) {
        Sensor sensor = sensorDao.findById(sensorId);
        if(sensor != null) {
            logger.info(String.format("Recognized sensor with ID: %s", sensorId));
            return sensor;
        } else {
            throw new UnknownSensorException(String.format("Received data from unknown sensor %s!", sensorId));
        }
    }

    @Override
    public void storeSensorValues(SensorData data) {
        if (data != null) {
            sensorDataDao.save(data);
        } else {
            throw new EmptySensorDataException("Received data are empty!");
        }
    }

    abstract public SensorData createSensorData(Sensor sensor, float data);
}
