package cz.kiele.homeautomation.webgui.controller;


import cz.kiele.homeautomation.webgui.rest.RestTemplateFactory;
import cz.kiele.homeautomation.webgui.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.web.csrf.CsrfToken;
import org.springframework.security.web.csrf.CsrfTokenRepository;
import org.springframework.web.util.UriUtils;
import org.springframework.web.util.WebUtils;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.security.NoSuchAlgorithmException;
import java.util.Base64;

public abstract class AbstractController {

    @Autowired
    RestTemplateFactory templateFactory;

    @Autowired
    UserService userService;

    @Autowired
    CsrfTokenRepository csrfTokenRepository;

    HttpHeaders basicAuthHeaders() {
        User user = (User) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        String email = user.getUsername();
        String password = null;
        try {
            password = userService.createSHA256Hash(user.getPassword());
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        HttpHeaders acceptHeaders = new org.springframework.http.HttpHeaders() {
            {
                set(org.springframework.http.HttpHeaders.ACCEPT,
                        MediaType.APPLICATION_JSON.toString());
            }
        };
        String authorization = email + ":" + password;
        String basic = new String(Base64.getEncoder().encode
                (authorization.getBytes(Charset.forName("US-ASCII"))));
        acceptHeaders.set("Authorization", "Basic " + basic);

        return acceptHeaders;
    }

    public HttpHeaders csrfHeaders() {
        CsrfToken csrfToken = csrfTokenRepository.generateToken(null);
        HttpHeaders headers = basicAuthHeaders();

        headers.add(csrfToken.getHeaderName(), csrfToken.getToken());
        headers.add("Cookie", "XSRF-TOKEN=" + csrfToken.getToken());

        return headers;
    }

    String encodeUrlPathSegment(String pathSegment,
                                HttpServletRequest httpServletRequest) {
        String enc = httpServletRequest.getCharacterEncoding();
        if (enc == null) {
            enc = WebUtils.DEFAULT_CHARACTER_ENCODING;
        }
        try {
            pathSegment = UriUtils.encodePathSegment(pathSegment, enc);
        } catch (UnsupportedEncodingException uee) {
        }
        return pathSegment;
    }

    public RestTemplateFactory getTemplateFactory() {
        return templateFactory;
    }

    public void setTemplateFactory(RestTemplateFactory templateFactory) {
        this.templateFactory = templateFactory;
    }
}
