package cz.kiele.homeautomation.webgui.controller;

import cz.kiele.homeautomation.webgui.data_objects.RoomDO;
import cz.kiele.homeautomation.webgui.enums.ActuatorType;
import cz.kiele.homeautomation.webgui.enums.RoomType;
import cz.kiele.homeautomation.webgui.model.Actuator;
import cz.kiele.homeautomation.webgui.model.Event;
import cz.kiele.homeautomation.webgui.model.Room;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.servlet.ModelAndView;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Controller
public class RoomController extends AbstractController {

    public static final String PATH_ROOM_DETAIL = "/room/";
    public static final String PATH_ADD_ROOM = "/addRoom";

    public static final String LIGHT_ON = "/lightOn/";
    public static final String LIGHT_OFF = "/lightOff/";



    @RequestMapping("/pages/room/{id}")
    public ModelAndView roomDetail(@PathVariable("id") int id){
        ModelAndView model = new ModelAndView();

        HttpEntity<RoomDO> entity = new HttpEntity<>(basicAuthHeaders());
        RestTemplate restTemplate = templateFactory.getObject();
        ResponseEntity<RoomDO> response = restTemplate.exchange(templateFactory.getTargetUri() + PATH_ROOM_DETAIL + id, HttpMethod.GET, entity, RoomDO.class);
        RoomDO room = response.getBody();

        model.addObject("roomDO", room);
        model.addObject("rules", room.getRules());
        model.addObject("sensors", room.getSensors());
        model.addObject("actuators", room.getActuators());
        model.addObject("roomId", "");

        model.setViewName("roomDetail");
        return model;
    }

    @RequestMapping("/pages/addRoom")
    public ModelAndView addRoomPage(){
        ModelAndView model = new ModelAndView();
        List<RoomType> types = new ArrayList<RoomType>( Arrays.asList(RoomType.values() ));



        model.addObject("room", new Room());
        model.addObject("types", types);

        model.setViewName("/admin/addRoom");
        return model;
    }

    @RequestMapping(value = "/addRoom", method = RequestMethod.POST)
    public String addRoom(@ModelAttribute("room")Room room, BindingResult result, ModelMap model){

        HttpEntity<Room> entity = new HttpEntity<>(basicAuthHeaders());
        RestTemplate restTemplate = templateFactory.getObject();
        ResponseEntity<Room> response = restTemplate.exchange(templateFactory.getTargetUri() + PATH_ADD_ROOM, HttpMethod.POST, entity, Room.class);

        return "redirect:/pages/setup/";
    }

    @RequestMapping(value = "/call/lightOn/{id}", method = RequestMethod.GET)
    public String roomLightOn(@PathVariable("id") int id){
        sendLightCommand(id, LIGHT_ON);

        return "redirect:/pages/room/" + id;
    }

    @RequestMapping(value = "/call/lightOff/{id}", method = RequestMethod.GET)
    public String roomLightOff(@PathVariable("id") int id){
        sendLightCommand(id, LIGHT_OFF);

        return "redirect:/pages/room/" + id;
    }

    private void sendLightCommand(int id, String commandUri) {
        HttpEntity<RoomDO> entity = new HttpEntity<>(basicAuthHeaders());
        RestTemplate restTemplate = templateFactory.getObject();
        ResponseEntity<RoomDO> response = restTemplate.exchange(templateFactory.getTargetUri() + PATH_ROOM_DETAIL + id, HttpMethod.GET, entity, RoomDO.class);
        RoomDO room = response.getBody();

        for (Actuator act : room.getActuators()) {
            if (act.getType().equals(ActuatorType.LIGHT_BULB) || act.getType().equals(ActuatorType.LIGHT)) {
                HttpEntity<Event> event = new HttpEntity<Event>(basicAuthHeaders());
                restTemplate.exchange(templateFactory.getTargetUri() + commandUri + act.getId(), HttpMethod.GET, event, String.class);
            }
        }
    }

}
